{% extends "base.html" %}
{% from "macros.html" import render_field, form_tag %}

{% block content %}
  {% call form_tag(form) %}
    <div class="row">
      <div class="col-md-9 col-sm-8 col-xs-12">
        <h1>{{tile or ("Education" if section_type == "EDU" else "Employment")}}</h1>
      </div>
    </div>
    <div class="row">
      <div class="col-md-6 col-sm-6 col-xs-12">
        {{ render_field(form.org_name) }}
        {{ render_field(form.city) }}
        {{ render_field(form.state) }}
        {{ render_field(form.country) }}
        {{ render_field(form.disambiguated_id) }}
      </div>
      <div class="col-md-6 col-sm-6 col-xs-12">
        {{ render_field(form.department) }}
        {{ render_field(form.role) }}
        {{ render_field(form.start_date) }}
        {{ render_field(form.end_date) }}
        {{ render_field(form.disambiguation_source) }}
        <button type="submit" class="btn btn-primary" value="Save">Save</button>
        <a role="button" class="btn btn-default" href="{{_url}}">Cancel</a>
      </div>
    </div>
  {% endcall %}
{% endblock %}
